library(logistf)
#zero inflated functions: ZIP and ZINB#
ZI<-function(y,x,model=c("zip","zinb")){
  model <- match.arg(model)   
  
  #'zip' zero-inflated-poisson
  #'zinb' zero-inflated-negative-binomial
  #vglm par settings
  control <- vglm.control(criterion="loglikelihood")
  control$min.criterion <- control$min.criterion[2]

  
  #vglm make model
  #ik <- mean(y)+sd(y)
  #ilambda<-mean(y[id])+sd(y[id])
  #iphi=.5
  
  #family selection
  family=switch(model,
          zip=zipoisson(),
          zinb=zinegbinomial())

#model frame selection          
mf <- model.frame(y~x+I(x^2),drop.unused.levels=T)
    
  mt <- attr(mf,"terms")
  y <- model.response(mf,"any")
  x <- model.matrix(mt,mf)
  w <- rep(1,nrow(mf))
  
  attr(x, "assign") = eval(VGAM:::attrassigndefault(x, mt))

#vglm.fit, catch the fail fittings as well
  test<-tryCatch({
     test <- VGAM:::vglm.fit(x=x,y=y,w=w,offset=0,Xm2=NULL,Ym2=NULL,
      control=control,function.name="vglm",criterion=control$criterion,
      
       family=family,
      
      etastart=NULL,mustart=NULL,coefstart=NULL,
      constraints=NULL,extra = list(), qr.arg = FALSE, Terms = mt
      )
  }
  , error=function(e) {return(NA)})

  return(test)
 	}



#zero truncated functions: HZTP and HZTNB#
 ZT_posnegb<-function(y,x,model=c("pps","pnegb")){
 	
  model <- match.arg(model)
  #model='pps', select positive poisson 
  #model='pnegb' select positive negativebinomial
  #vglm par settings
  control <- vglm.control(criterion="loglikelihood")
  control$min.criterion <- control$min.criterion[2]

  
  #vglm make model
  id<-which(y>0)
  ilambda<-mean(y[id])+sd(y[id])
  family=switch(model,"pps"=pospoisson(ilambda=ilambda),
          "pnegb"=posnegbinomial())
  mf <- model.frame(y[id]~x[id]+I(x[id]^2),drop.unused.levels=T)
  mt <- attr(mf,"terms")
  y <- model.response(mf,"any")
  x <- model.matrix(mt,mf)
  w <- rep(1,nrow(mf))
  
  attr(x, "assign") = eval(VGAM:::attrassigndefault(x, mt))

  test<-tryCatch({
     test <- VGAM:::vglm.fit(x=x,y=y,w=w,offset=0,Xm2=NULL,Ym2=NULL,
      control=control,function.name="vglm",criterion=control$criterion,
      
       family=family,
      
      etastart=NULL,mustart=NULL,coefstart=NULL,
      constraints=NULL,extra = list(), qr.arg = FALSE, Terms = mt
      )
  }
  , error=function(e) {return(NA)})

  return(test)
 	}
 	
#LR function (4.1) by using parallel computing#
sfLibrary(VGAM)
sfExport(list=list("ZT_posnegb","ZI"))
LR<-function(alpha,sp,envar,TYPE=c("pps","pnegb","zip","zinb")){	
TYPE<-match.arg(TYPE)
	  del.col <- which(colSums(sp!=0) < 3)
    if (length(del.col) > 0) {sp <- sp[,-del.col]}
    alpha<-alpha/sqrt(sum(alpha^2))
 	error.id<-c()
 	site <- as.matrix(envar)%*%alpha
 	design <- model.matrix(~site)
 	
    ztp<-switch(TYPE,
    pps=sfApply(sp,2,function(y) ZT_posnegb(y,x=site,model='pps')),
    pnegb=sfApply(sp,2,function(y) ZT_posnegb(y,x=site,model='pnegb')),
    zip=sfApply(sp,2,function(y) ZI(y,x=site,model='zip')),
    zinb=sfApply(sp,2,function(y) ZI(y,x=site,model='zinb')))
    
    error.id<- !is.na(ztp)
    ztp<-ztp[error.id]
        sp<-sp[,error.id]
 	
 	Eztp<-matrix(0,nrow=nrow(sp),ncol=length(ztp))
 	
     for (j in 1:length(ztp)){
     	if(TYPE=='pps'||TYPE=='pnegb')
     	{
 	 id<-which(sp[,j]>0)
     Eztp[id,j]<-ztp[[j]]$fitted.values
     }
     else Eztp[,j]<-ztp[[j]]$fitted.values
     }
 
 	cs<-colSums(sp)
 	rs<-rowSums(sp)
 	wt<-cs/sum(cs)
 	response.log<-response<-Eztp
 	response.all<-rowSums(response)
 	
 	nozero <- response !=0
    response.log[nozero]<-log(response[nozero])
    lr<-sum(sp*response.log)-sum(rs*log(response.all))-sum(log(wt)*cs)
    return(-lr)
	}

#LR function with ordianry Poisson#
	LR_pois <- function(alpha,sp.new,envar){
	alpha<-alpha/sqrt(sum(alpha^2))
    site <- as.matrix(envar)%*%alpha
    design <- model.matrix(~site+I(site^2))
    response.log <- response <- sfApply(sp.new,2,function(x){
        glm.fit(design,x,family=poisson())$fitted.values
    })

    wt<-colSums(sp.new)/sum(sp.new)#species weights
    response.all<-rowSums(response)

    # avoid ifelse, as that one is terribly slow
    nozero <- response !=0
    response.log[nozero]<-log(response[nozero])

    lr<-sum(sp.new*response.log)-sum(rowSums(sp.new)*log(response.all))-sum(log(wt)*colSums(sp.new))
    return(-lr)
	}
#LR with logistic regression#
LR.binomial<-function(alpha,sp,envar,geovar=NULL,gamma=NULL,exact=F,...){	

	del.col <- which(colSums(sp!=0) < 3)
    if (length(del.col) > 0) {
    	sp <- sp[,-del.col]
    }
    alpha<-alpha/sqrt(sum(alpha^2))
 	error.id<-c()
 	site1 <- as.matrix(envar)%*%alpha
if(exact){
	zbi<-apply(sp,2,function(y) logistf(y~site1+I(site1^2)))	
}
else zbi<-apply(sp,2,function(y) glm(y~site1+I(site1^2),family=binomial('logit')))	 	
# fit exact logistic regression
#zbi<-apply(sp,2,function(y) logistf(y~site1+I(site1^2)))

     error.id<- !is.na(zbi)
    zbi<-zbi[error.id]
    sp<-sp[,error.id]
 	
 	Ezbi<-matrix(0,nrow=nrow(sp),ncol=length(zbi))
 	
     for (j in 1:length(zbi)){
 
#extract fitted values from exact logistic regression
      		if (exact) Ezbi[,j]<-zbi[[j]]$predict   
     		else Ezbi[,j]<-zbi[[j]]$fitted.values  
     }
 
 	cs<-colSums(sp)
 	rs<-rowSums(sp)
 	wt<-cs/sum(cs)
 	response.log<-response<-Ezbi

response.log<-log(response)
response.log.com<-log(1-response)
pool.log<-log(response%*%wt)
pool.log.com<-log(1-response%*%wt)

lr<-sum(sp*response.log)+sum((1-sp)*response.log.com)-sum(rowSums(sp)*pool.log)-sum(rowSums(1-sp)*pool.log.com)
 return(-lr)
	}	

#####################
#graphical functions#
#####################

#ordination diagram#
mybiplot<-function (x, y, var.axes = TRUE, col, cex = rep(par("cex"), 2), 
    xlabs = NULL, ylabs = NULL, expand = 1, xlim = NULL, ylim = NULL, 
    arrow.len = 0.1, main = NULL, sub = NULL, xlab = NULL, ylab = NULL, 
    ...) 
{
    n <- nrow(x)
    p <- nrow(y)
    if (missing(xlabs)) {
        xlabs <- dimnames(x)[[1L]]
        if (is.null(xlabs)) 
            xlabs <- 1L:n
    }
    xlabs <- as.character(xlabs)
    dimnames(x) <- list(xlabs, dimnames(x)[[2L]])
    if (missing(ylabs)) {
        ylabs <- dimnames(y)[[1L]]
        if (is.null(ylabs)) 
            ylabs <- paste("Var", 1L:p)
    }
    ylabs <- as.character(ylabs)
    dimnames(y) <- list(ylabs, dimnames(y)[[2L]])
    if (length(cex) == 1L) 
        cex <- c(cex, cex)
    if (missing(col)) {
        col <- par("col")
        if (!is.numeric(col)) 
            col <- match(col, palette(), nomatch = 1L)
        col <- c(col, col + 1L)
    }
    else if (length(col) == 1L) 
        col <- c(col, col)
    unsigned.range <- function(x) c(-abs(min(x, na.rm = TRUE)), 
        abs(max(x, na.rm = TRUE)))
    rangx1 <- unsigned.range(x[, 1L])
    rangx2 <- unsigned.range(x[, 2L])
    rangy1 <- unsigned.range(y[, 1L])
    rangy2 <- unsigned.range(y[, 2L])
    if (missing(xlim) && missing(ylim)) 
        xlim <- ylim <- rangx1 <- rangx2 <- range(rangx1, rangx2)
    else if (missing(xlim)) 
        xlim <- rangx1
    else if (missing(ylim)) 
        ylim <- rangx2
    ratio <- max(rangy1/rangx1, rangy2/rangx2)/expand
    on.exit(par(op))
    op <- par(pty = "s")
    if (!is.null(main)) 
        op <- c(op, par(mar = par("mar") + c(0, 0, 1, 0)))
    plot(x, type = "n", xlim = xlim, ylim = ylim, col = col[1L], 
        xlab = xlab, ylab = ylab, sub = sub, main = main, ...)
        abline(v=0,h=0,col='black',lty=3)
    text(x, xlabs, cex = cex[1L], col = 'black', ...)
    par(new = TRUE)
    plot(y, axes = FALSE, type = "n", xlim = xlim * ratio, ylim = ylim * 
        ratio, xlab = "", ylab = "", col = col[2L], ...)
    #axis(3, col = col[2L], ...)
    #axis(4, col = col[2L], ...)
   
    box(col = col[1L])
    text(y[, 1L]*1.5, y[, 2L]*1.5 , labels = ylabs, cex = cex[2L], col = 'blue', ...)
    if (var.axes) 
        arrows(0, 0, y[, 1L] , y[, 2L] , col = 'blue', 
            length = arrow.len)
    #points(z,pch=1:12)   
    #legend(1,1,spName,pch=19,col=coolcol,cex=0.6,bg='grey90',text.col='blue',xjust=0,ncol=2)  
    
    invisible()
} 
#other functions for output format#
latexTable<-function(x){
ta<-lapply(x,function(x) x$code)
id<-which(ta==1)
len<-length(id)
temp<-sapply(x[id],function(x) x$estimate)
temp1<-apply(temp,2,function(x) x/sqrt(sum(x^2)))
sdAll<-apply(temp1,1,sd)
meanAll<-rowSums(temp1)/len
return(cbind(meanAll,sdAll))
}


#####
afAna<-function(DatPar,main,nameG){
id<-which(is.na(DatPar))
if (length(id)==0){DatPar<-DatPar}
else (DatPar<-DatPar[-id])
ztp.ztp1<-lapply(DatPar,function(x) x['estimate'])
ztp.ztp2<-matrix(unlist(ztp.ztp1),nrow=1000-length(id),ncol=13,byrow=TRUE)
ztp.ztp<-apply(ztp.ztp2,1,function(x) x/sqrt(sum(x^2)))#13x1000
#AvePar<-rowSums(ztp.ztp)/1000
varS<-apply(ztp.ztp,1,function(x) sd(x))
return(varS)
#return(AvePar)
	}
 